package ru.dbotthepony.mc.otm.core.util

import net.minecraft.core.HolderLookup
import net.minecraft.core.registries.Registries
import net.minecraft.nbt.CompoundTag
import net.minecraft.nbt.Tag
import net.minecraft.resources.ResourceKey
import net.minecraft.resources.ResourceLocation
import net.minecraft.server.level.ServerLevel
import net.minecraft.util.RandomSource
import net.minecraft.world.item.ItemStack
import net.minecraft.world.level.block.state.BlockState
import net.minecraft.world.level.storage.loot.LootParams
import net.minecraft.world.level.storage.loot.LootTable
import net.minecraft.world.level.storage.loot.parameters.LootContextParamSets
import net.minecraft.world.level.storage.loot.parameters.LootContextParams
import ru.dbotthepony.mc.otm.block.entity.MatteryBlockEntity.Companion.LOOT_TABLE_KEY
import ru.dbotthepony.mc.otm.block.entity.MatteryBlockEntity.Companion.LOOT_TABLE_SEED_KEY
import ru.dbotthepony.mc.otm.core.otmRandom

class BlockLootTableHolder(private val listener: Runnable = Runnable {  }) {
	private var ignoreListener = false

	var lootTable: ResourceKey<LootTable>? = null
		set(value) {
			if (field != value) {
				field = value
				if (!ignoreListener) listener.run()
			}
		}

	var lootTableSeed: Long? = null
		set(value) {
			if (field != value) {
				field = value
				if (!ignoreListener) listener.run()
			}
		}

	fun save(nbt: CompoundTag, registry: HolderLookup.Provider) {
		try {
			ignoreListener = true

			if (lootTable != null) {
				nbt.putString(LOOT_TABLE_KEY, lootTable!!.location().toString())

				if (lootTableSeed != null)
					nbt.putLong(LOOT_TABLE_SEED_KEY, lootTableSeed!!)
			}
		} finally {
			ignoreListener = false
		}
	}

	fun load(nbt: CompoundTag, registry: HolderLookup.Provider) {
		try {
			ignoreListener = true

			if (nbt.contains(LOOT_TABLE_KEY, Tag.TAG_STRING.toInt())) {
				lootTable = ResourceLocation.tryParse(nbt.getString(LOOT_TABLE_KEY))?.let { ResourceKey.create(Registries.LOOT_TABLE, it) }
				lootTableSeed = if (nbt.contains(LOOT_TABLE_SEED_KEY, Tag.TAG_LONG.toInt())) nbt.getLong(LOOT_TABLE_SEED_KEY) else null
			}
		} finally {
			ignoreListener = false
		}
	}

	fun lookup(level: ServerLevel): LootTable {
		val lootTable = lootTable ?: return LootTable.EMPTY
		return level.server.reloadableRegistries().getLootTable(lootTable)
	}

	private fun selectRandom(overrideSeed: Long?, fallback: RandomSource): RandomSource {
		val lootTableSeed = overrideSeed ?: lootTableSeed
		return if (lootTableSeed == null) fallback else GJRAND64RandomSource(lootTableSeed)
	}

	fun getItems(params: LootParams, level: ServerLevel, overrideSeed: Long? = null): MutableList<ItemStack> {
		return lookup(level).getRandomItems(params, selectRandom(overrideSeed, level.otmRandom))
	}

	fun getItems(params: LootParams.Builder, blockState: BlockState, overrideSeed: Long? = null): MutableList<ItemStack> {
		val level = params.level
		val create = params.withParameter(LootContextParams.BLOCK_STATE, blockState).create(LootContextParamSets.BLOCK)
		return getItems(create, level, overrideSeed)
	}
}
